#ifndef MITM_H
#define MITM_H

#include <Arduino.h>
#include "config.h" // Include if needed for specific pins or settings
#include "ui.h"     // For ButtonPressType, maybe UIState if needed internally

// --- MitM Attack States ---
typedef enum {
    MITM_STATE_IDLE,         // Not active, ready to be configured/started
    MITM_STATE_SELECT_TARGET,// <<< UI state for selecting the target AP
    MITM_STATE_STARTING,     // Setting up Rogue AP, DNS, Web Server, Deauth
    MITM_STATE_RUNNING,      // Actively waiting for clients and credentials
    MITM_STATE_CAPTURED,     // Credentials have been captured
    MITM_STATE_VIEW_LOGS,    // <<< New state to view captured passwords
    MITM_STATE_STOPPING      // Cleaning up resources
} MitmState;

// --- Function Declarations ---

// Initialization (called once in setup)
void mitm_init();

// Start the MitM attack process (potentially takes target AP info)
// void mitm_start(const AccessPointInfo& targetAP); // Example if using scanner data
void mitm_start(); // Simpler start for now

// Stop the MitM attack and clean up
void mitm_stop();

// Main update loop (called repeatedly in loop when in MITM_MODE)
void mitm_update();

// Draw the MitM UI on the OLED
void mitm_draw();

// Handle button input when in MITM_MODE
void mitm_handle_input(ButtonPressType pressType);

// Get the current internal state of the MitM module
MitmState mitm_get_current_state(); // Optional getter

#endif // MITM_H 